﻿#include "pch.h"
#include "CharacterFormattingDemoRenderer.h"
#include "Common/DirectXHelper.h"

using namespace CharacterFormattingDemo;

using namespace D2D1;
using namespace Microsoft::WRL;

CharacterFormattingDemoRenderer::CharacterFormattingDemoRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources) : 
    m_deviceResources(deviceResources),
    m_useCustomRenderer(false)
{
    // Create device independent resources
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
        );

    ComPtr<IDWriteTextFormat> textFormat;

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateTextFormat(
            L"Times New Roman",
            nullptr,
            DWRITE_FONT_WEIGHT_NORMAL,
            DWRITE_FONT_STYLE_NORMAL,
            DWRITE_FONT_STRETCH_NORMAL,
            24.0f,
            L"en-us",
            &textFormat)
        );

    std::wstring text = L"This paragraph of text rendered with "
                        L"DirectWrite is based on "
                        L"IDWriteTextFormat and IDWriteTextLayout "
                        L"objects and is capable of numerous types "
                        L"of character formatting, including "
                        L"italic, bold, underline, strikethrough, "
                        L"and using different font sizes and "
                        L"families, such as Courier New "
                        L"or the ever-popular Comic Sans MS.";

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateTextLayout(
            text.c_str(),
            (uint32) text.length(),
            textFormat.Get(),
            440.0f,         // Max width for paragraph
            std::numeric_limits<float>::infinity(),
            &m_textLayout)
        );

    // Set character formatting
    DWRITE_TEXT_RANGE textRange;

    std::wstring strFind = L"IDWriteTextFormat";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetFontStyle(DWRITE_FONT_STYLE_ITALIC, textRange)
        );

    strFind = L"IDWriteTextLayout";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetFontStyle(DWRITE_FONT_STYLE_ITALIC, textRange)
        );

    strFind = L"italic";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetFontStyle(DWRITE_FONT_STYLE_ITALIC, textRange)
        );

    strFind = L"bold";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetFontWeight(DWRITE_FONT_WEIGHT_BOLD, textRange)
        );

    strFind = L"underline";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetUnderline(true, textRange)
        );

    strFind = L"strikethrough";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetStrikethrough(true, textRange)
        );

    strFind = L"sizes";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetFontSize(48.0f, textRange)
        );

    strFind = L"Courier New";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetFontFamilyName(strFind.data(), textRange)
        );

    strFind = L"Comic Sans MS";
    textRange.startPosition = text.find(strFind.data());
    textRange.length = strFind.length();
    DX::ThrowIfFailed(
        m_textLayout->SetFontFamilyName(strFind.data(), textRange)
        );

    DX::ThrowIfFailed(
        m_textLayout->GetMetrics(&m_textMetrics)
        );

    // Instantiate BasicTextRenderer
    m_textRenderer = new BasicTextRenderer();

    CreateDeviceDependentResources();
}

void CharacterFormattingDemoRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(ColorF(ColorF::Black), &m_blackBrush)
        );
}
void CharacterFormattingDemoRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
}

void CharacterFormattingDemoRenderer::Update(DX::StepTimer const& timer)
{
}

// Renders a frame to the screen.
void CharacterFormattingDemoRenderer::Render()
{
    ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
    Windows::Foundation::Size logicalSize = m_deviceResources->GetLogicalSize();

    context->SaveDrawingState(m_stateBlock.Get());
    context->BeginDraw();
    context->Clear(ColorF(ColorF::AliceBlue));

    // Center text on the screen
    Matrix3x2F screenTranslation = Matrix3x2F::Translation(
        (logicalSize.Width - m_textLayout->GetMaxWidth()) / 2,
        (logicalSize.Height - m_textMetrics.height) / 2);

    context->SetTransform(screenTranslation * 
                m_deviceResources->GetOrientationTransform2D());

    // Display paragraph of text
    D2D1_POINT_2F origin = Point2F();

    if (!m_useCustomRenderer)
    {
        context->DrawTextLayout(origin,
                                m_textLayout.Get(),
                                m_blackBrush.Get());
    }
    else
    {
        DrawingContext drawingContext(context, 
                                      m_blackBrush.Get());

        m_textLayout->Draw(&drawingContext, m_textRenderer.Get(), origin.x, origin.y);
    }


    // Ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    HRESULT hr = context->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    context->RestoreDrawingState(m_stateBlock.Get());
}

void CharacterFormattingDemoRenderer::SetUseCustomRenderer(bool use)
{
    m_useCustomRenderer = use;
}


